require 'libmagellan'

require 'logger'


module Libmagellan

  class Core

    DEFAULT_LOG_LEVEL = Logger::INFO

    def initialize(*args)
      # HTTP
      @http = HTTP.new(*args.dup)

      # MQTT
      options = args.last.is_a?(Hash) ? args.pop : {}
      mqtt_options = options.dup
      if mqtt_options[:mqtt_host].present? and mqtt_options[:mqtt_port].present?
        mqtt_options[:host] = mqtt_options.delete(:mqtt_host)
        mqtt_options[:port] = mqtt_options.delete(:mqtt_port)
        mqtt_options[:secure] = mqtt_options.delete(:mqtt_secure) || false
        @mqtt = MQTT.new(mqtt_options)
        @mqtt.logger = logger
      end
    end


    ##
    ## HTTP
    ##

    def request(*args, &block)
      @http.request(*args, &block)
    end
    alias :req :request

    def ping
      @http.ping
    end

    def test_ping(*args)
      @http.test_ping(*args)
    end

    def test(*args, &block)
      @http.test(*args, &block)
    end

    def json_body(r)
      @http.json_body(r)
    end



    ##
    ## MQTT
    ##

    def mqtt_token(path, method=:get, body="", headers={})
      r = request(path, method, body, headers)
      if r.code.to_i >= 200 and r.code.to_i < 300
        token = r['authorization']
        @mqtt.token = token if @mqtt
        token
      else
        r
      end
    end

    def publish(*args)
      @mqtt.publish(*args)
    end
    alias :pub :publish

    def subscribe(*args, &block)
      @mqtt.subscribe(*args, &block)
    end
    alias :sub :subscribe

    def unsubscribe(*args)
      @mqtt.unsubscribe(*args)
    end
    alias :unsub :unsubscribe

    def get_packet(topic=nil, &block)
      @mqtt.get_packet(topic, &block)
    end

    def disconnect
      @mqtt.disconnect
    end

    def set_will(topic, payload, retain=false, qos=0)
      @mqtt.set_will(topic, payload, retain, qos)
    end

    def get_message(*args, &block)
      @mqtt.get(*args, &block)
    end


    def logger
      @logger ||= Proc.new {
        logger = ::Logger.new(STDOUT)
        logger.level = DEFAULT_LOG_LEVEL
        logger
      }.call
    end

    def logger=(logger_)
      @logger = logger_
    end

    # Set logger log level
    # @param [Integer] level Log level
    def log_level=(level)
      logger.level = level
    end

    def inspect
      h = @http
      m = @mqtt
      arr = ["\#<#{self.class}: @base_uri=\"#{h.base_uri.to_s}\"",
             "@consumer_key=\"#{h.consumer_key}\"",
             "@client_version=\"#{h.client_version}\"",]
      arr += ["@mqtt_host=\"#{m.host}\"",
              "@mqtt_port=#{m.port}"] if @mqtt
      arr << ">"
      arr.join(", ")
    end

  end

end
